<?php

App::uses('CakeEventListener', 'Event');

class EvBasketCheckForDiscountListener implements CakeEventListener {

	public function implementedEvents() {
		return array(
			'EvDiscount.Component.DiscountCodes.codeAdded' => 'addDiscount',
			'EvBasket.Component.Basket.itemAdd' => 'rebuildDiscount',
			'EvBasket.Component.Basket.itemUpdated' => 'rebuildDiscount',
			'EvBasket.Component.Basket.itemDeleted' => 'rebuildDiscount',

			'EvLoyaltyPoints.Component.Points.usePoints' => 'addLoyaltyDiscount',
		);
	}

	/**
	 * queue up the confirmation emails
	 *
	 * @param   CakeEvent
	 */
	public function addDiscount(CakeEvent $Event, $data = null) {
		$dataCode = $Event->data['data'];
		// Event subject will be the EvDiscount.DiscountCodesComponent
		return $this->rebuildDiscount($Event, $dataCode);
	}

	/**
	 * rebuild discount due to basket amends
	 *
	 * @param   CakeEvent
	 */
	public function rebuildDiscount(CakeEvent $Event, $dataCode = null) {
		if (! CakePlugin::loaded('EvDiscount')) {
			return false;
		}

		App::uses('DiscountLib', 'EvDiscount.Lib');

		$EventSubject = $Event->subject();
		$EventSubject->_controller->loadModel('EvDiscount.DiscountCode');
		if (! is_object($EventSubject->_controller->DiscountCookie)) {
			$EventSubject->_controller->DiscountCookie = $EventSubject->_controller->loadComponent('EvDiscount.DiscountCookie');
		}

		// Get the basket data to find the discount codes
		$codes = array();
		if ($dataCode != null) {
			$codes[$dataCode['DiscountCode']['code']] = $dataCode;
		} else {
			$basketData = $EventSubject->_controller->BasketManager->getBasketData();

			if (!empty($basketData)) {
				foreach ($basketData as $key => $value) {
					if ($value['name'] == 'discount_code') {
						$codes[$value['data']] = $value;
					}
				}
			}
		}

		// The cookie isn't used any more but there can still be multiple discount codes added to the basket in the
		// database potentially. Current functionality only allows one discount code at a time though.

		if (! is_object($EventSubject->_controller->BasketManager)) {
			$EventSubject->_controller->BasketManager = $EventSubject->_controller->loadComponent('EvBasket.BasketManager');
		}

		if (! empty($codes) && is_array($codes)) {

			$validCodes = $EventSubject->_controller->DiscountCode->validateCodes($codes);

			if (! empty($validCodes)) {
				// Discount codes were found - now check to see if any of them have any restrictions that need checking.
				//
				// If a coupon has restrictions, the discount plugin cant handle it - its going to need to be handed off
				// to whatever is creating the restriction (i.e you've edited the config file to set what model you're
				// restricting on, so you now need to create an event to handle that restriction as its not possible for the
				// discount plugin to do it).
				//
				// This gets complicated as hell because of how it was written, but the theory behind it is to prevent you
				// ever needing to touch the discount plugin.
				//
				// The problem now comes that because you hand off to another event listener, you then need to completely
				// skip everything else that the discount plugin does, which seems stupid but unless someones got a better
				// way of doing this whilst retaining the flexability of not being restricted to using one shop plugin it's
				// the only way.
				//
				// So, the below code - it sends a copy of the discount code and the basket to an event.
				//
				// What gets returned is a modified basket, with the price reduced on whatever basket item(s) the discount
				// applied to. This is pretty restrictive as you then have no idea what discount was applied where. There's
				// curently nothing that can be done about this as there was no provision made to store discount information
				// on an order other than the total discount amount.
				foreach ($validCodes as $code => $codeData) {
					if (isset($codeData['CodeRestriction']) && ! empty($codeData['CodeRestriction'])) {
						// This code has restrictions that need checking
						$discount = new CakeEvent('EvBasket.Component.Basket.CheckDiscountRestrictions', $EventSubject->_controller, array(
							'discount' => $codeData,
							'basket' => $EventSubject->_controller->BasketManager->getBasket()
						));
						$EventSubject->_controller->getEventManager()->dispatch($discount);

						$results = $discount->result;

						if (! empty($results) && is_array($results)) {
							$EventSubject->_controller->BasketManager->updateItem($results, null, null, 0, false);
						}

						// Remove this discount from the list as it cant be handled down below now.
						unset($validCodes[$code]);
					}
				}
			}

		}
		$total = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.subtotal')
		);

		$grandTotal = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.grandtotal')
		);

		$taxTotal = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.tax')
		);

		$discount = 0;
		$discountCode = null;

		//Default discount position after subtotal. Discount will always contain what is to betaken off of the subtotal.
		//Tax amounts always contain the discounted value so display this after discount.
		$discountSequence = 15;

		if (! empty($validCodes) && is_array($validCodes)) {
			foreach ($validCodes as $codeData) {
				if (
					! empty($codeData['DiscountCode']['min_order']) &&
					$total['amount'] < $codeData['DiscountCode']['min_order']
				) {
					return false;
				}

				$amount = DiscountLib::calculate($codeData, $total['amount']);

				if (! empty($amount) && Configure::read('SiteSetting.ev_basket.discounts_include_tax') == '1' && $codeData['DiscountCode']['discount_type_id'] == 1) {
					$basketTaxRate = number_format($taxTotal['amount'] / ($grandTotal['amount'] - $taxTotal['amount']), 2);

					if ($basketTaxRate == -1) {
						$discountTax = 0;
					} else {
						$discountTax = number_format($amount - ($amount / (1 + $basketTaxRate)), 2);
					}

					$discountPreTax = $amount - $discountTax;
					$discount += $discountPreTax;
				} else {
					$discount += $amount;
				}

				$discountCode = $codeData['DiscountCode']['code'];
			}
		}
		if (! empty($discount) && $discount > 0) {
			$discount = number_format($discount, 2, ".", "");

			$EventSubject->_controller->BasketManager->manageTotalRow(
				Configure::read('EvBasket.labels.discount'),
				-$discount,
				$discountSequence,
				false,
				-$discount
			);

			//Don't update the tax as that is dealt with when rebuilding totals

			// Add a basket data record to handle the discount row
			$EventSubject->_controller->BasketManager->setBasketData('discount_code', $discountCode, true);

			$Event->subject()->_controller->BasketManager->rebuildTotals();
		}

		return true;
	}

	public function addLoyaltyDiscount(CakeEvent $Event) {
		if (! CakePlugin::loaded('EvLoyaltyPoints')) {
			return false;
		}

		$EventSubject = $Event->subject();

		if (! is_object($EventSubject->_controller->BasketManager)) {
			$EventSubject->_controller->BasketManager = $EventSubject->_controller->loadComponent('EvBasket.BasketManager');
		}

		$total = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.subtotal')
		);

		App::uses('PointLib', 'EvLoyaltyPoints.Lib');

		if ($Event->data['pointsToUse'] <= 0) {
			$discount = 0;
		} else {
			$discount = PointLib::calculate($total['amount'], 'EvBasket', 0, $Event->data['pointsToUse']);
		}

		$EventSubject->_controller->BasketManager->manageTotalRow(
			Configure::read('EvBasket.labels.discount'),
			-$discount,
			15
		);

		// Add a basket data record to handle the loyalty disount row
		$EventSubject->_controller->BasketManager->setBasketData('loyalty_points_used', $Event->data['pointsToUse'], true);

		$Event->subject()->_controller->BasketManager->rebuildTotals();

		return true;
	}
}
